/*
アイデア・ハック!! Arduinoで遊ぼう（11）
サーボモーターで「自動給餌器」を作ろう！
									2014.05.16　三月兎
									URL:robot-fan.net
*/
#include <Servo.h> 

//-----------------------
// 定数の定義
//-----------------------
Servo myservo;     // サーボオブジェクトを宣言 
int servopin = 9;   //サーボモーターの信号線 接続ピン
//long time = 3;  
//long count = 0;

const int SW_COUNT = 3;           // スイッチの数
const int SW[SW_COUNT] = {3,4,5}; // 各スイッチの入力ピン番号
//スイッチ0～2の定義
const int SW_UP = 0;
const int SW_START = 1;
const int SW_DOWN = 2;

//スイッチの回路は負論理なので、ONがLOWになる
const int SW_ON = LOW;
const int SW_OFF = HIGH;

//LCDエスケープシーケンス
char LCD_CLR[2] = {0x1b,'C'};              //クリア

char LCD_CURSOR[4] = {0x1b,'G','@','@'};   //カーソル位置設定(x, y)
//LCDカーソル位置を表す文字
char LCD_CURSOR_POSITION[16] = {'@','A','B','C'
                               ,'D','E','F','G'
                               ,'H','I','J','K'
                               ,'L','M','N','O'};
//LCD１行目（左）の表示領域
const int LCD_1L_POS_X = 0;
const int LCD_1L_POS_Y = 0;
const int LCD_1L_LEN = 8;
//LCD１行目（右）の表示領域
const int LCD_1R_POS_X = 11;
const int LCD_1R_POS_Y = 0;
const int LCD_1R_LEN = 5;
//LCD２行目の表示領域
const int LCD_2L_POS_X = 0;
const int LCD_2L_POS_Y = 1;
const int LCD_2L_LEN = 16;

//プログラムのモード
const int MODE_SW_INPUT = 0;                //スイッチ入力モード
const int MODE_TIMER_COUNT = 1;             //タイマーカウントモード
//タイマーの最大/最小
const int TIME_MAX = 720;                    //60min * 12h                 
const int TIME_MIN = 1;  
//１分間（ミリ秒）
const unsigned long ONE_MINUITE = 60L * 1000L;

//LCDブリンクON/OFF
const int BLINK_ON = 1;
const int BLINK_OFF = 0;

//LCDブリンク時間
const unsigned long BLINK_ON_TIME = 700L;
const unsigned long BLINK_OFF_TIME = 300L;

//-----------------------
// グローバル変数の定義
//-----------------------
int G_mode = MODE_SW_INPUT;                          //プログラムのモード
int G_old_val[SW_COUNT] = {SW_OFF, SW_OFF, SW_OFF};  //前回のスイッチの値
int G_blink_stat = BLINK_ON;                         //ブリンク状態
unsigned long G_time = TIME_MIN;                     //タイマーの時間
unsigned long G_sw_on_time[SW_COUNT];                //スイッチを押し続けている時間
unsigned long G_start_time;                          //スタートボタンを押してからの時間
unsigned long G_blink_time;                          //ブリンク時間

void setup() {
    //サーボモーターの設定
    pinMode(servopin, OUTPUT);	   // ピンモードを出力に設定
    myservo.attach(servopin);      // 9番ピンをサーボオブジェクトに接続 
    myservo.write(0);              // サーボモーターのシャフトを0角度に指定

  //全てのスイッチを入力モードに設定
    for(int i = 0; i < SW_COUNT; i++) {
       pinMode(SW[i], INPUT);
    }
    //LCDとのシリアル通信を開始
    Serial.begin(9600);
    //LCDクリア
    write_escape_sequence(LCD_CLR, 2);
}
void loop() {
    int sw_no;
    unsigned long current_time;
    //プログラムのモードによって処理を振り分ける
    switch(G_mode) {
        case MODE_SW_INPUT:
            //スイッチ入力モード
            sw_no = input_time();
            switch(sw_no) {
                case SW_UP:
                case SW_DOWN:
                    //スタートメッセージ領域をクリア
                    disp_start_message(" ");
                    //メッセージ領域をクリア
                    disp_message(" ");
                    break;
                case SW_START:
                    if(G_time < TIME_MIN) {
                        G_time = TIME_MIN;
                    }
                    G_start_time = millis();
                    G_blink_time = G_start_time;
                    //スタートメッセージを表示
                    disp_start_message("Start");
                    // メッセージを表示
                    disp_message("Hold on.");
                    //タイマーカウントモードにする
                    G_mode = MODE_TIMER_COUNT;
                    break;
            }
            G_blink_stat = BLINK_ON;
            break;
        case MODE_TIMER_COUNT:
            //タイマーカウントモード
            current_time = millis();
            if(current_time - G_start_time >= ONE_MINUITE) {
                //１分経過
                G_time--;
                if(G_time <= 0) {
                    //終了メッセージを表示
                    disp_start_message("End");
                    // メッセージを表示
                    disp_message("Come and get it!");
                    
                    for(int pos = 0; pos <= 80; pos += 1) // 0～80まで 1ずつ 
                    {
                    //サーボ出力
                      myservo.write(pos);              // シャフトの角度を指示 
                      delay(30);                       // 30ms待機 
                    } 
                    
                    //ブリンクを停止
                    G_blink_stat = BLINK_OFF;
                    //スイッチ入力モードにする
                    G_mode = MODE_SW_INPUT;
                }
                G_start_time = current_time;
                G_blink_time = G_start_time;
                G_blink_stat = BLINK_ON;
            }
            //実行中と分かるように時間をブリンクさせる
            if(G_blink_stat == BLINK_ON) {
                if(current_time - G_blink_time >= BLINK_ON_TIME) {
                    G_blink_stat = BLINK_OFF;
                    G_blink_time = current_time;
                }
            }
            else {
                if(current_time - G_blink_time >= BLINK_OFF_TIME) {
                    G_blink_stat = BLINK_ON;
                    G_blink_time = current_time;
                }
            }
            break;
    }
    //時間を表示する（ブリンクあり）
    disp_time(G_time, G_blink_stat);
}

//-----------------------------------
//スイッチで時間を入力する
//-----------------------------------
int input_time() {
    int int_swNumber = -1;          //何も押してないときの番号を設定
    int sw_val;
    //スイッチ１～３の状態を取得
    for(int i = 0; i < SW_COUNT; i++) {
        sw_val = digitalRead(SW[i]);      //スイッチの状態を読み取る
		//スイッチを押した瞬間
        if(sw_val == SW_ON && G_old_val[i] == SW_OFF) {
            int_swNumber = i;             //スイッチの番号を取得
            G_sw_on_time[i] = millis();	  //押した瞬間の時間を取得
        } 
	//スイッチを押し続けているとき
        if(sw_val == SW_ON && G_old_val[i] == SW_ON) {
            if(millis() - G_sw_on_time[i] > 500L) {
                int_swNumber = i;
            }
        } 
        G_old_val[i] = sw_val;            //スイッチの値を保存 
    }
    //スイッチの入力があった時
    switch(int_swNumber) {
      case SW_UP:
          if(G_time < TIME_MAX) {
              G_time++;
          }
          break;
      case SW_DOWN:
          if(G_time > TIME_MIN) {
              G_time--;
          }
          break;
      case SW_START:
          break;
    }
    return int_swNumber;
}
//-----------------------------------
//LCDに時間を表示する
//-----------------------------------
void disp_time(int time, int blink_stat) {
    //表示位置を設定する（1行目左）
    set_cursor(LCD_1L_POS_X, LCD_1L_POS_Y);
    //時間を表示する
    if(blink_stat == BLINK_ON) {
        Serial.print(time);
        Serial.print("min");
    }
    //不要な領域をクリアする
    int spcCount;
    if(blink_stat == BLINK_ON) {
        if(time < 10) {
            spcCount = LCD_1L_LEN - 4;
        }
        else {
            spcCount = LCD_1L_LEN - 5;
        }
    }
    else {
        spcCount = LCD_1L_LEN;
    }
    for(int i = 0; i < spcCount; i++) {
        Serial.print(" ");
    }
}
//-----------------------------------
//LCDにスタートメッセージを表示する
//-----------------------------------
void disp_start_message(char message[]) {
    //表示位置を設定する（1行目右）
    set_cursor(LCD_1R_POS_X, LCD_1R_POS_Y);
    //クリアする
    for(int i = 0; i < LCD_1R_LEN; i++) {
        Serial.print(" ");
    }
    //表示位置を設定する（1行目右）
    set_cursor(LCD_1R_POS_X, LCD_1R_POS_Y);
    //メッセージを表示する
    Serial.print(message);
}
//-----------------------------------
//LCDにメッセージを表示する
//-----------------------------------
void disp_message(char message[]) {
    //表示位置を設定する（2行目左）
    set_cursor(LCD_2L_POS_X, LCD_2L_POS_Y);
    //クリアする
    for(int i = 0; i < LCD_2L_LEN; i++) {
        Serial.print(" ");
    }
    //表示位置を設定する（2行目左）
    set_cursor(LCD_2L_POS_X, LCD_2L_POS_Y);
    //メッセージを表示する
    Serial.print(message);
}
//-----------------------------------
//LCDの表示位置を設定する
//-----------------------------------
void set_cursor(int x, int y) {
    //LCDの表示位置設定用のエスケープシーケンスを組み立てて出力する
    LCD_CURSOR[2] = LCD_CURSOR_POSITION[x];
    LCD_CURSOR[3] = LCD_CURSOR_POSITION[y];
    write_escape_sequence(LCD_CURSOR, 4);
}
//-----------------------------------
//LCDにエスケープシーケンスを出力する
//-----------------------------------
void write_escape_sequence(char escseq[], int length) {
    //LCDにエスケープシーケンスを2進数で出力する
    for(int i = 0; i < length; i++) {
        Serial.write(escseq[i]);
    }
}


